%% plot epsr/sigma contour
function figure5(Parm, N, T1, T2, Iobj, Epsr, Sigm)
%%
% reconstructed
nx = Parm.nxv;
ny = Parm.nyv;
nz = Parm.nzv;
epsr = Parm.epsr(1) * ones(nx, ny, nz);
sigm = Parm.sigm(1) * ones(nx, ny, nz);
for n = 1 : N
	i = Iobj(n, 1);
	j = Iobj(n, 2);
	k = Iobj(n, 3);
	epsr(i, j, k) = T2(n) + Parm.epsr(1);
	sigm(i, j, k) = T1(n) + Parm.sigm(1);
end

% plot
%plotX(3 : 6, Parm, Epsr, epsr);
%plotY(9 : 12, Parm, Epsr, epsr);
plotZ(2 : 3, Parm, Epsr, epsr);

% debug
%plotZ1(Parm, Epsr, Sigm, epsr, sigm);
%plotZ2(Parm, epsr);
end

%% X=const
function plotX(i_range, Parm, Epsr, epsr)
%%
% phantom
fp = zeros(Parm.nyp, Parm.nzp);

% reconstructed
nx = Parm.nxv;
ny = Parm.nyv;
nz = Parm.nzv;
fr = zeros(ny, nz);

figure(5);
tiledlayout(length(i_range), 2);
%ifig = 1;

for i = i_range
	ip = round((i - 0.5) * Parm.H);
	if (ip > Parm.nxp) || (i > nx)
		continue;
	end
	x = (i - 0.5) * Parm.H * Parm.D * 1e3;

	% phantom
	nexttile;
	%subplot(2, length(i_range), ifig);

	for k = 1 : Parm.nzp
	for j = 1 : Parm.nyp
		fp(j, k) = Epsr(ip, j, k);
	end
	end

	g = pcolor(fp'); g.EdgeColor = 'none';
	clim(Parm.epsr_scale);
	axis equal;
	axis tight;
	colorbar;
	xlabel('Y[mm]');
	ylabel('Z[mm]');
  	title(sprintf('比誘電率(数値ファントム), X=%gmm', x));

	% reconstructed
	nexttile;
	%subplot(2, length(i_range), length(i_range) + ifig); ifig = ifig + 1;
	
	for k = 1 : nz
	for j = 1 : ny
		fr(j, k) = epsr(i, j, k);
	end
	end

	contourf(fr');
	clim(Parm.epsr_scale);
	axis equal;
	axis tight;
	colorbar;
	xlabel(sprintf('Y[mm]/%g', Parm.H));
	ylabel(sprintf('Z[mm]/%g', Parm.H));
  	title(sprintf('比誘電率(再構成値), X=%gmm', x));
end
end

%% Y=const
function plotY(j_range, Parm, Epsr, epsr)
%%
% phantom
fp = zeros(Parm.nxp, Parm.nzp);

% reconstructed
nx = Parm.nxv;
ny = Parm.nyv;
nz = Parm.nzv;
fr = zeros(nx, nz);

figure(5);
tiledlayout(length(j_range), 2);
%ifig = 1;

for j = j_range
	jp = round((j - 0.5) * Parm.H);
	if (jp > Parm.nyp) || (j > ny)
		continue;
	end
	y = (j - 0.5) * Parm.H * Parm.D * 1e3;

	% phantom
	nexttile;
	%subplot(2, length(j_range), ifig);

	for k = 1 : Parm.nzp
	for i = 1 : Parm.nxp
		fp(i, k) = Epsr(i, jp, k);
	end
	end

	g = pcolor(fp'); g.EdgeColor = 'none';
	clim(Parm.epsr_scale);
	axis equal;
	axis tight;
	colorbar;
	xlabel('X[mm]');
	ylabel('Z[mm]');
  	title(sprintf('比誘電率(数値ファントム), Y=%gmm', y));

	% reconstructed
	nexttile;
	%subplot(2, length(j_range), length(j_range) + ifig); ifig = ifig + 1;
	
	for k = 1 : nz
	for i = 1 : nx
		fr(i, k) = epsr(i, j, k);
	end
	end

	contourf(fr');
	clim(Parm.epsr_scale);
	axis equal;
	axis tight;
	colorbar;
	xlabel(sprintf('X[mm]/%g', Parm.H));
	ylabel(sprintf('Z[mm]/%g', Parm.H));
  	title(sprintf('比誘電率(再構成値), Y=%gmm', y));
end
end

%% Z=const
function plotZ(k_range, Parm, Epsr, epsr)
%%
% phantom
fp = zeros(Parm.nxp, Parm.nyp);

% reconstructed
nx = Parm.nxv;
ny = Parm.nyv;
nz = Parm.nzv;
fr = zeros(nx, ny);

figure(5);
tiledlayout(length(k_range), 2);
%ifig = 1;

for k = k_range
	kp = round((k - 0.5) * Parm.H);
	if (kp > Parm.nzp) || (k > nz)
		continue;
	end
	z = (k - 0.5) * Parm.H * Parm.D * 1e3;

	% phantom
	nexttile;
	%subplot(2, length(k_range), ifig);

	for j = 1 : Parm.nyp
	for i = 1 : Parm.nxp
		fp(i, j) = Epsr(i, j, kp);
	end
	end

	g = pcolor(fp'); g.EdgeColor = 'none';
	clim(Parm.epsr_scale);
	axis equal;
	axis tight;
	colorbar;
	xlabel('X[mm]');
	ylabel('Y[mm]');
  	title(sprintf('比誘電率(数値ファントム), Z=%gmm', z));

	% reconstructed
	nexttile;
	%subplot(2, length(k_range), length(k_range) + ifig); ifig = ifig + 1;
	
	for j = 1 : ny
	for i = 1 : nx
		fr(i, j) = epsr(i, j, k);
	end
	end

	contourf(fr');
	clim(Parm.epsr_scale);
	axis equal;
	axis tight;
	colorbar;
	xlabel(sprintf('X[mm]/%g', Parm.H));
	ylabel(sprintf('Y[mm]/%g', Parm.H));
  	title(sprintf('比誘電率(再構成値), Z=%gmm', z));
end
end

%% Z=const (debug : 比誘電率と導電率の比較)
function plotZ1(Parm, Epsr, Sigm, epsr, sigm)
%%
strtitle = ["比誘電率(数値ファントム)", "導電率(数値ファントム) [S/m]", "比誘電率(再構成値)", "導電率(再構成値) [S/m]"];
climit = [Parm.epsr_scale; Parm.sigm_scale; Parm.epsr_scale; Parm.sigm_scale];

% phantom
f1 = zeros(Parm.nxp, Parm.nyp);
f2 = zeros(Parm.nxp, Parm.nyp);

% reconstructed
nx = Parm.nxv;
ny = Parm.nyv;
f3 = zeros(nx, ny);
f4 = zeros(nx, ny);

%for k = 1 : nz
for k = 2 : 2  % edit here
	kp = round((k - 0.5) * Parm.H);
	if (kp > Parm.nzp)
		continue;
	end
	for j = 1 : Parm.nyp
	for i = 1 : Parm.nxp
		f1(i, j) = Epsr(i, j, kp);
		f2(i, j) = Sigm(i, j, kp);
	end
	end

	for j = 1 : ny
	for i = 1 : nx
		f3(i, j) = epsr(i, j, k);
		f4(i, j) = sigm(i, j, k);
	end
	end

	strfig = sprintf('Figure 5 : Z=%gmm', (k - 0.5) * Parm.H * Parm.D * 1e3);
	figure('Name', strfig, 'NumberTitle', 'off');
	tiledlayout(2, 2);

	for m = 1 : 4
		nexttile;
		if     (m == 1)
			img = f1;
		elseif (m == 2)
			img = f2;
		elseif (m == 3)
	    	img = f3;      
		elseif (m == 4)
			img = f4;
			end
		if (m == 1) || (m == 2)
			g = pcolor(img'); g.EdgeColor = 'none';	%g.FaceColor = 'interp';
			%imagesc(img');
			strx = 'X[mm]';
			stry = 'Y[mm]';
		elseif (m == 3) || (m == 4)
			contourf(img');
			strx = sprintf('X[mm]/%g', Parm.H);
			stry = sprintf('Y[mm]/%g', Parm.H);
		end
		clim(climit(m, :));
		axis equal;
		axis tight;
		colorbar;
		xlabel(strx);
		ylabel(stry);
   		title(strtitle(m));
	end
end
end

%% Z=const (debug : 等高線図の種類)
function plotZ2(Parm, epsr)
%%
% reconstructed
nx = Parm.nxv;
ny = Parm.nyv;
fr = zeros(ny, nx);

k = 5;
for j = 1 : ny
for i = 1 : nx
	fr(j, i) = epsr(i, j, k);
end
end

figure(5);
tiledlayout(2, 2);

for m = 1 : 4
	nexttile;
	if     (m == 1)
		contourf(fr);
		title('contourf');
	elseif (m == 2)
		contour(fr);
		title('contour');
	elseif (m == 3)
		g = pcolor(fr); g.EdgeColor = 'none';
		title("g = pcolor(img); g.EdgeColor = 'none'");
	elseif (m == 4)
		g = pcolor(fr); g.EdgeColor = 'none'; g.FaceColor = 'interp';
		title("g = pcolor(img); g.EdgeColor = 'none'; g.FaceColor = 'interp'");
	end
	clim(Parm.epsr_scale);
	axis equal;
	axis tight;
	colorbar;
	xlabel(sprintf('X[mm]/%g', Parm.H));
	ylabel(sprintf('Y[mm]/%g', Parm.H));
end
end
