% green's function (multi frequency)
function [G, Ga] = green(Parm, N, Na, Nf, Robj)
%%
rant = Parm.rant;
iant = Parm.iant;

G  = zeros(3 * N,  3 * N, Nf);
Ga = zeros(    Na, 3 * N, Nf);

for ifreq = 1 : Nf
	A = Parm.A(ifreq);
	C = Parm.C(ifreq);
	k = Parm.k(ifreq);
	G(:, :, ifreq) = green_1(N, A, C, k, Robj);    % 3N x 3N
	Gall = green_2(N, Na, A, k, Robj, rant);       % 3Na x 3N
	for ir = 1 : Na
		Ga(ir, :, ifreq) = Gall(iant(ir) * Na + ir, :);
	end
end

end

% オブジェクト内のボクセル同士のグリーン関数
function G = green_1(N, A, C, k0, Robj)
%%
Gxx = zeros(N, N);
Gyy = zeros(N, N);
Gzz = zeros(N, N);
Gxy = zeros(N, N);
Gxz = zeros(N, N);
Gyx = zeros(N, N);
Gyz = zeros(N, N);
Gzx = zeros(N, N);
Gzy = zeros(N, N);
	
for n = 1 : N
	dx = Robj(:, 1) - Robj(n, 1);
	dy = Robj(:, 2) - Robj(n, 2);
	dz = Robj(:, 3) - Robj(n, 3);
	Rmn = sqrt(dx.^2 + dy.^2 + dz.^2);
	Rmn = max(Rmn, eps);  % avoid zero division
	amn = k0 * Rmn;
	cxmn = dx ./ Rmn;
	cymn = dy ./ Rmn;
	czmn = dz ./ Rmn;

	% 同一偏波間のグリーン関数
	Gxx(:, n) = A * green_f(1, cxmn, cxmn, amn);
	Gyy(:, n) = A * green_f(1, cymn, cymn, amn);
	Gzz(:, n) = A * green_f(1, czmn, czmn, amn);

	% 異なる偏波間のグリーン関数
	Gxy(:, n) = A * green_f(0, cxmn, cymn, amn);
	Gxz(:, n) = A * green_f(0, cxmn, czmn, amn);
	Gyz(:, n) = A * green_f(0, cymn, czmn, amn);
	Gyx(:, n) = A * green_f(0, cymn, cxmn, amn);
	Gzx(:, n) = A * green_f(0, czmn, cxmn, amn);
	Gzy(:, n) = A * green_f(0, czmn, cymn, amn);
	%
end

% 同一偏波間の対角成分
% 異なる偏波間の対角成分は0
for n = 1 : N
	Gxx(n, n) = C;
	Gyy(n, n) = C;
	Gzz(n, n) = C;
end
	
G = [Gxx Gxy Gxz; Gyx Gyy Gyz; Gzx Gzy Gzz];
end

% オブジェクト内のボクセルとアンテナ間のグリーン関数
function Gall = green_2(N, Na, A, k0, Robj, rant)	
%%
Gxx = zeros(Na, N);
Gxy = zeros(Na, N);
Gxz = zeros(Na, N);
Gyx = zeros(Na, N);
Gyy = zeros(Na, N);
Gyz = zeros(Na, N);
Gzx = zeros(Na, N);
Gzy = zeros(Na, N);
Gzz = zeros(Na, N);

for n = 1 : N
%for m = 1 : Na
	dx = rant(:, 1) - Robj(n, 1);
	dy = rant(:, 2) - Robj(n, 2);
	dz = rant(:, 3) - Robj(n, 3);
	Rmn = sqrt(dx.^2 + dy.^2 + dz.^2);
	amn = k0 * Rmn;
	cxmn = dx ./ Rmn;
	cymn = dy ./ Rmn;
	czmn = dz ./ Rmn;

	% 同一偏波間のグリーン関数
	Gxx(:, n) = A * green_f(1, cxmn, cxmn, amn);
	Gyy(:, n) = A * green_f(1, cymn, cymn, amn);
	Gzz(:, n) = A * green_f(1, czmn, czmn, amn);
 
	% 異なる偏波間のグリーン関数
	Gxy(:, n) = A * green_f(0, cxmn, cymn, amn);
	Gxz(:, n) = A * green_f(0, cxmn, czmn, amn);
	Gyz(:, n) = A * green_f(0, cymn, czmn, amn);
	Gyx(:, n) = A * green_f(0, cymn, cxmn, amn);
	Gzx(:, n) = A * green_f(0, czmn, cxmn, amn);
	Gzy(:, n) = A * green_f(0, czmn, cymn, amn);
%end
end

Gall = [Gxx Gxy Gxz; Gyx Gyy Gyz; Gzx Gzy Gzz];
end

%% eq. (3-20) (array version)
function g = green_f(b, cpmn, cqmn, amn)
%%
g = exp(-1i * amn) ./ amn.^3 ...
  .* (b * (amn.^2 - 1i * amn - 1) - (cpmn .* cqmn) .* (amn.^2 - 3i * amn - 3));
end
