%% input parameters
function [Parm, Na, Nf] = input_parameter()
%%
Parm.restart = 0;    % restart (0/1)
Parm.H = 8;          % ボクセル合体数(整数)

% 計算条件
Parm.maxit = 100;    % 最大反復回数
Parm.tol = 1e-4;     % 収束判定条件
Parm.alpha = 0.01;   % 正則化パラメータα
Parm.c0 = 3.0;       % コントラスト初期値c0

% アンテナ
ra = 60e-3;          % アンテナ面と原点の距離[m]
nxa = 4;             % X方向アンテナ数
nya = 4;             % Y方向アンテナ数
nza = 3;             % Z方向アンテナ数

% (1)周囲 (2)脂肪 (3)乳腺 (4)カップ (5)がん (6)皮膚
%Parm.epsr = [6.2  7.0  35.0  9.4 50.0 35.0 ];
%Parm.sigm = [0.15 0.21  1.05   0  1.5  1.05];
Parm.epsr = [5.0  6.5 40.0 9.4 54.0 40.0];
Parm.sigm = [0.1  0.2  1.0   0  1.3  1.0];
Parm.epsr(1) = Parm.epsr(2);  % background=fat
Parm.sigm(1) = Parm.sigm(2);
Parm.epsr(4) = Parm.epsr(2);  % cup=fat
Parm.sigm(4) = Parm.sigm(2);

% 周波数
freq = 2.0e9;   % 1周波数のとき
%freq = linspace(2.0e9, 2.2e9, 2);  % 複数周波数のとき
Nf = length(freq);

% scale (plot)
Parm.epsr_scale = [0 60];
Parm.sigm_scale = [0 1.45];

% 定数
Parm.R = 48e-3;      % カップ半径[m]
Parm.D = 1e-3;       % 数値ファントム単位長[m]（1mm以外はテストしていない）
eps0 = 8.8541878e-12;
mu0 = pi * 4e-7;

omega = 2 * pi * freq;
Parm.oe = omega * eps0;
Parm.joe = 1i * Parm.oe;
cer_b = Parm.epsr(1) + Parm.sigm(1) ./ Parm.joe;
Parm.k = omega .* sqrt(mu0 * eps0 * cer_b);
%Parm.k = omega * sqrt(mu0 * eps0 * Parm.epsr(1));

rc = (Parm.H * Parm.D) * (0.75 / pi)^(1 / 3);   % セルの等価半径[m]
jom = 1i * omega * mu0;
jka = 1i * Parm.k * rc;
% A : 5.207の係数の一部
Parm.A = -jom .* Parm.k * (Parm.H * Parm.D)^3 / (4 * pi);
% C : 5.210よりtau(r)を除き+1
Parm.C = (2 * exp(-jka) .* (1 + jka) - 3) ./ (3 * Parm.joe .* cer_b);

% antenna
[Na, x, y, z, d] = antenna(ra, nxa, nya, nza);
Parm.rant = [x' y' z'];
Parm.iant = d;
end

%% setup antenna parameters
function [Na, x, y, z, d] = antenna(ra, nx, ny, nz)
%%
%{
% debug
Na = 1;
x(1) = +ra;
y(1) = 0;
z(1) = 25e-3;
d(1) = 2; % Z
%}
%
Na = (nx * ny) + (2 * nz) * (nx + ny);
x = zeros(1, Na);
y = zeros(1, Na);
z = zeros(1, Na);
d = zeros(1, Na);

dx = 2 * ra / nx;
dy = 2 * ra / ny;
dz = 1 * ra / nz;
n = 0;

% +X
for k = 1 : nz
for j = 1 : ny
	n = n + 1;
	x(n) = +ra;
	y(n) = -ra + (j - 0.5) * dy;
	z(n) =       (k - 0.5) * dz;
	%d(n) = 2;  % Z
	d(n) = mod(j + k, 2) + 1;  % Y or Z
end
end
%
% -X
for k = 1 : nz
for j = 1 : ny
	n = n + 1;
	x(n) = -ra;
	y(n) = -ra + (j - 0.5) * dy;
	z(n) =       (k - 0.5) * dz;
	%d(n) = 2;  % Z
	d(n) = mod(j + k, 2) + 1;  % Y or Z
end
end
%
% +Y
for k = 1 : nz
for i = 1 : nx
	n = n + 1;
	x(n) = -ra + (i - 0.5) * dx;
	y(n) = +ra;
	z(n) =       (k - 0.5) * dz;
	%d(n) = 2;  % X
	d(n) = 2 * mod(i + k, 2);  % X or Z
end
end
%
% -Y
for k = 1 : nz
for i = 1 : nx
	n = n + 1;
	x(n) = -ra + (i - 0.5) * dx;
	y(n) = -ra;
	z(n) =       (k - 0.5) * dz;
	%d(n) = 2;  % X
	d(n) = 2 * mod(i + k, 2);  % X or Z
end
end

% +Z
for j = 1 : ny
for i = 1 : nx
	n = n + 1;
	x(n) = -ra + (i - 0.5) * dx;
	y(n) = -ra + (j - 0.5) * dy;
	z(n) = ra;
	%d(n) = 1;   % Y
	d(n) = mod(i + j, 2);  % X or Y
end
end

% error check
if (n ~= Na)
	fprintf('*** invalid antennas : %d, %d\n', n, Na);
end
for ir = 1 : Na
	if (d(ir) < 0) || (d(ir) > 2)
		fprintf('*** invalid polarization : #%d\n', ir);
	end
end
%
end
