%{
mom_inv.m (chapter 5_2_1)
%}

clearvars;
%close all;
tic;

% input parameters
[Parm, Na, Nf] = input_parameter();
%figure1(Parm, Na); shg;

% read numerical phantom
medLabFile = 'mediumLabelList_Reso1mm_Case1.txt';
[Parm, Epsr, Sigm] = read_phantom(medLabFile, Parm);
if Parm.err
	return;
end

% setup
[Parm, N, T10, T20, Robj, Iobj] = setup(Parm, Epsr, Sigm);
strtitle = sprintf('H=%d, N=%d, 3N=%d, Na=%d, Na^2=%d, Nf=%d, α=%g, c0=%g', Parm.H, N, 3 * N, Na, Na^2, Nf, Parm.alpha, Parm.c0);
fprintf("%s\n", strtitle);
I = eye(3 * N);

% green's function
% G  : 3N x 3N x Nf
% Ga : Na x 3N x Nf
[G, Ga] = green(Parm, N, Na, Nf, Robj);

% E-inc
Ei = einc(Parm, N, Na, Nf, Robj);    % 3N x Na x Nf

% forward calculation
if (Parm.restart == 0)
	% solve E
	E_fwd = zeros(Na^2, Nf);
	for ifreq = 1 : Nf
		tau = T10 + (T20 * Parm.joe(ifreq));   % 1 x N
		S = diag([tau; tau; tau]);             % 3N x 3N
		[Et, E_fwd(:, ifreq)] = solve(ifreq, N, Na, S, G, Ga, Ei);
		%figure2(Parm, N, Iobj, Et); shg;
	end

	% initial value for iteration
	T1 = zeros(N, 1);
	T2 = zeros(N, 1);
	for n = 1 : N
		T1(n) = Parm.c0 * 0.03;
		T2(n) = Parm.c0;
	end
elseif (Parm.restart == 1)
	% restart : load
	load 'mom_inv.mat' E_fwd T1 T2
end

% backward calculation (iteration)
fprintf('iter    dE/E        dεr/εr       dσ/σ      (εr-εr0)/εr0   (σ-σ0)/σ0      g\n');
fiter = [];
for iter = 1 : Parm.maxit
	sumf = zeros(1, 6);
	for ifreq = 1 : Nf
		% solve E
		% Et    : 3N x Na
		% E_bwd : Na^2 x 1
		% S     : 3N x 3N
		tau = T1 + (T2 * Parm.joe(ifreq));
		S = diag([tau; tau; tau]);
		[Et, E_bwd] = solve(ifreq, N, Na, S, G, Ga, Ei);

		% dE : Na^2 x 1
		dE = E_bwd - E_fwd(:, ifreq); 

		% matrix D
		D = zeros(Na^2, 3 * N);                % Na^2 x 3N
		invISG = inv(I - S * G(:, :, ifreq));  % 3N x 3N
		for ir = 1 : Na
			D((ir - 1) * Na + 1 : ir * Na, :) = Ga(:, :, ifreq) * invISG * diag(Et(:, ir));  % Na x 3N
		end

		% g : regularization factor
		tr = trace(D' * D);
		eerr = norm(dE) / norm(E_bwd);
		g = Parm.alpha * tr / (3 * N) * eerr^2;

		% update T
		dm = (D' * D + g * I) \ D';    % 3N x Na^2
		dS = dm * dE;                  % 3N x 1
		dS = reshape(dS, N, 3);        % N x 3
		dT = (dS(:, 1) + dS(:, 2) + dS(:, 3)) / 3;
		T1 = T1 - real(dT);
		T2 = T2 - imag(dT) / Parm.oe(ifreq);
		
		% correct T
		[T1, T2] = correct(T1, T2, Parm);

		% monitor
		sumf(1) = sumf(1) + eerr;
		sumf(2) = sumf(2) + norm(imag(dT)) / norm(T2) / Parm.oe(ifreq);
		sumf(3) = sumf(3) + norm(real(dT)) / norm(T1);
		sumf(4) = sumf(4) + norm(T2 - T20) / norm(T20);
		sumf(5) = sumf(5) + norm(T1 - T10) / norm(T10);
		sumf(6) = sumf(6) + g;
	end
	sumf = sumf / Nf;

	% monitor
	if (iter == 1)
		fid = fopen('mom_inv.log', 'wt');
	end
	fprintf(     '%3d %e %e %e %e %e %e\n', iter, sumf);
	fprintf(fid, '%3d %e %e %e %e %e %e\n', iter, sumf);

	% plot iteration
	fiter = figure3(strtitle, fiter, sumf); shg;
	figure4(Parm, N, T10, T20, T1, T2); shg;

	% check convergence
	if (sumf(2) < Parm.tol) && (sumf(3) < Parm.tol)
		break;
	end
end

% plot epsr/sigma contour and histogram
figure5(Parm, N, T1, T2, Iobj, Epsr, Sigm);
figure6(Parm, T1, T2);

% output
save 'mom_inv.mat' E_fwd T1 T2

toc
%end

%% solve E
function [Et, Es] = solve(ifreq, N, Na, S, G, Ga, Ei)
%%
I = eye(3 * N);
igs = I - G(:, :, ifreq) * S;        % 3N x 3N
Et = igs \ Ei(:, :, ifreq);          % 3N x Na
es = Ga(:, :, ifreq) * S * Et;       % Na x Na
Es = reshape(es, Na^2, 1);           % Na^2 x 1
end

%% correct T
function [T1, T2] = correct(T1, T2, Parm)
%%
sgm0 = T1 + Parm.sigm(1);
eps0 = T2 + Parm.epsr(1);
eps1 = max(1, min(60, eps0));
sgm1 = max(0.02 * eps1, min(0.04 * eps1, sgm0));
T1 = sgm1 - Parm.sigm(1);
T2 = eps1 - Parm.epsr(1);
end
