%% read numerical phantom file
function [Parm, Epsr, Sigm] = read_phantom(fn, Parm)
%%
Epsr = [];
Sigm = [];

% file open
[fid, errmsg] = fopen(fn, 'rt');
Parm.err = (fid < 0);
if Parm.err
	fprintf('%s : %s\n', errmsg, fn);
	return;
end
	
fgetl(fid);  % dummy=1[mm]

nxf = str2double(fgetl(fid));  % =99
nzf = str2double(fgetl(fid));  % =50
nyf = str2double(fgetl(fid));  % =96

Parm.nxp = nxf + 1;  % 100
Parm.nyp = nyf + 4;  % 100
Parm.nzp = nzf;      % 50

tmap = zeros(Parm.nxp, Parm.nyp, Parm.nzp);
Epsr =  ones(Parm.nxp, Parm.nyp, Parm.nzp);
Sigm = zeros(Parm.nxp, Parm.nyp, Parm.nzp);

% tissue map
for j = 1 : nyf
for k = 1 : nzf
	tline = strtrim(fgetl(fid));
	token = sscanf(tline, '%d');
	tmap(1 : nxf, j + 1, k) = token;  % Y + 1 !!
end
end

% correct Y=98
tmap(:, 98, :) = tmap(:, 2, :);

fclose(fid);

% set Epsr and Sigm
for k = 1 : Parm.nzp
for j = 1 : Parm.nyp
for i = 1 : Parm.nxp
	id = tmap(i, j, k);  % =0/1/2/3/4/5
	Epsr(i, j, k) = Parm.epsr(id + 1);
	Sigm(i, j, k) = Parm.sigm(id + 1);
end
end
end

%{
%case1 : 0:250151 1:192970 2:24291 3:29113 4:3475 total=500000 (500000)
%case2 : 0:250151 1:177097 2:39234 3:29113 4:4405 total=500000 (500000)
%case3 : 0:250151 1:178534 2:41562 3:29113 4:640 total=500000 (500000)
num = zeros(1, 5);
for k = 1 : Parm.nzp
for j = 1 : Parm.nyp
for i = 1 : Parm.nxp
	num(tmap(i, j, k) + 1) = num(tmap(i, j, k) + 1) + 1;
end
end
end
fprintf('0:%d 1:%d 2:%d 3:%d 4:%d total=%d (%d)\n', num(1), num(2), num(3), num(4), num(5), sum(num), Parm.nxp * Parm.nyp * Parm.nzp);
%}

% plot X/Y/Z cross section
%ispan = 36 : 2 : 64;
%plotX(ispan, Parm.nzp, Parm.nyp, tmap, [0 4]);
%plotX(ispan, Parm.nzp, Parm.nyp, Epsr, Parm.epsr_scale);
%plotX(ispan, Parm.nzp, Parm.nyp, Sigm, Parm.sigm_scale);
%jspan = 36 : 2 : 64;
%plotY(jspan, Parm.nzp, Parm.nxp, tmap, [0 4]);
%plotY(jspan, Parm.nzp, Parm.nxp, Epsr, Parm.epsr_scale);
%plotY(jspan, Parm.nzp, Parm.nxp, Sigm, Parm.sigm_scale);
kspan = 10 : 0 : 10;
%plotZ(kspan, Parm.nyp, Parm.nxp, tmap, [0 4]);
%plotZ(kspan, Parm.nyp, Parm.nxp, Epsr, Parm.epsr_scale);
%plotZ(kspan, Parm.nyp, Parm.nxp, Sigm, Parm.sigm_scale);
end

% plot X cross section
function plotX(ispan, nz, ny, f, scale)
%%
p = zeros(nz, ny);
for i = ispan
	nexttile;
	for j = 1 : ny
	for k = 1 : nz
		p(k, j) = f(i, j, k);
	end
	end
	%contourf(p);
	s = pcolor(p); s.EdgeColor = 'none';
	clim(scale);
	title(sprintf('X=%gmm', i - 0.5));
	xlabel("Y[mm]");
	ylabel("Z[mm]");
	colorbar;
	axis equal;
	axis tight;
	view(2);
end
end

% plot Y cross section
function plotY(jspan, nz, nx, f, scale)
%%
p = zeros(nz, nx);
for j = jspan
	nexttile;
	for i = 1 : nx
	for k = 1 : nz
		p(k, i) = f(i, j, k);
	end
	end
	%contourf(p);
	s = pcolor(p); s.EdgeColor = 'none';
	clim(scale);
	title(sprintf('Y=%gmm', j - 0.5));
	xlabel("X[mm]");
	ylabel("Z[mm]");
	colorbar;
	axis equal;
	axis tight;
	view(2);
end
end

% plot Z cross section
function plotZ(kspan, ny, nx, f, scale)
%%
p = zeros(ny, nx);
for k = kspan  % edit here
	nexttile;
	for j = 1 : ny
	for i = 1 : nx
		p(j, i) = f(i, j, k);
	end
	end
	%contourf(p);
	s = pcolor(p); s.EdgeColor = 'none';
	clim(scale);
	title(sprintf('Z=%gmm', k - 0.5));
	xlabel("X[mm]");
	ylabel("Y[mm]");
	colorbar;
	axis equal;
	axis tight;
	view(2);
end
end
