%% setup
function [Parm, N, T10, T20, Robj, Iobj] = setup(Parm, Epsr, Sigm)
%%
x0 = (-Parm.nxp / 2 + Parm.H / 2) * Parm.D;  % [m]
x1 = (+Parm.nxp / 2 - Parm.H / 2) * Parm.D;
y0 = (-Parm.nyp / 2 + Parm.H / 2) * Parm.D;
y1 = (+Parm.nyp / 2 - Parm.H / 2) * Parm.D;
z0 =      (       0 + Parm.H / 2) * Parm.D;
z1 =      (Parm.nzp - Parm.H / 2) * Parm.D;
dh = Parm.H * Parm.D;  % [m]
xn = x0 : dh : x1;  % [m]
yn = y0 : dh : y1;
zn = z0 : dh : z1;
nx = length(xn);
ny = length(yn);
nz = length(zn);

% No. of objects
Parm.nxv = nx;
Parm.nyv = ny;
Parm.nzv = nz;

ei = zeros(nx, ny, nz);
si = zeros(nx, ny, nz);
for k = 1 : nz
for j = 1 : ny
for i = 1 : nx
	xs = (i - 1) * Parm.H + 1 : i * Parm.H;
	ys = (j - 1) * Parm.H + 1 : j * Parm.H;
	zs = (k - 1) * Parm.H + 1 : k * Parm.H;
	ei(i, j, k) = sum(sum(sum(Epsr(xs, ys, zs)))) / Parm.H^3;
	si(i, j, k) = sum(sum(sum(Sigm(xs, ys, zs)))) / Parm.H^3;
end
end
end
%nexttile; contourf(si(:, :, 1)); colorbar; axis equal;
%nexttile; contourf(si(:, :, 2)); colorbar; axis equal;
%nexttile; contourf(si(:, :, 3)); colorbar; axis equal;
%nexttile; contourf(si(:, :, 4)); colorbar; axis equal;

% contrast
r0 = Parm.R * 1.0;  % [m]
n = 0;
for k = 1 : nz
for j = 1 : ny
for i = 1 : nx
	r = sqrt(xn(i)^2 + yn(j)^2 + zn(k)^2);
	if (r < r0)
		n = n + 1;
	end
end
end
end
N = n;
Iobj = zeros(N, 3);
Robj = zeros(N, 3);
T10  = zeros(N, 1);
T20  = zeros(N, 1);

n = 0;
for k = 1 : nz
for j = 1 : ny
for i = 1 : nx
	r = sqrt(xn(i)^2 + yn(j)^2 + zn(k)^2);
	if (r < r0)
		n = n + 1;
		Iobj(n, 1) = i;
		Iobj(n, 2) = j;
		Iobj(n, 3) = k;
		Robj(n, 1) = xn(i);  % [m]
		Robj(n, 2) = yn(j);
		Robj(n, 3) = zn(k);
		T10(n) = si(i, j, k) - Parm.sigm(1);
		T20(n) = ei(i, j, k) - Parm.epsr(1);
		%fprintf('%d %g %g %g %g\n', n, xn(i), yn(j), zn(k), r);
	end
end
end
end
%plot3(Robj(:, 1), Robj(:, 2), Robj(:, 3), 'o'); xlabel('X[m]'); ylabel('Y[m]'); zlabel('Z[m]'); axis([-Parm.R Parm.R -Parm.R Parm.R 0 Parm.R]); axis equal; grid;
end
